/*
 * counter_app_ant.c
 *
 *  This is the ANT application code.
 *
 */
#include <stdio.h>  // printf/puts
#include <string.h> // strncmp
#include "lib_ant.h"
#include "lib_dispatcher.h"
#include "counter_app_ant.h"
#include "LPC11xx.h"
#include "system.h"
#include "timer32.h"
#include "hal_io.h"

typedef enum
{
   ANT_IDLE = 0,
   ANT_SETUP_SET_CHANNEL_ID,
   ANT_SETUP_SET_CHANNEL_PERIOD,
   ANT_SETUP_SET_CHANNEL_SEARCH_TIMEOUT,
   ANT_SETUP_SET_TX_POWER,
   ANT_SETUP_SET_RADIO_FREQUENCY,
   ANT_SETUP_OPEN_CHANNEL,
   ANT_RUN,
   ANT_COMPLETE,
   ANT_FAILURE
} ant_state_t;



ant_state_t state                         = ANT_IDLE;
ant_state_t next_state;


static uint8_t           send_buffer[MESG_DATA_SIZE - 1];
static volatile uint8_t  counter          = 0;
static volatile uint32_t sent_packets     = 0, retransmitted_tx_packets = 0;
static volatile uint32_t received_packets = 0, retransmitted_rx_packets = 0;
static volatile uint32_t lost_packets     = 0, invalid_counter_count = 0;
static volatile uint32_t discovery_time   = 0;
static volatile bool     test_timed_out   = false, received_first_packet = false;


void on_process_ant_counter_app(uint8_t size, uint8_t *buffer)
{
   static bool lp_search_timeout_set = false;

   if (strncmp((const char *)&buffer[0], "reset", 5) == 0)
   {
      reset_ant_radio();
      ANT_ResetSystem();
      state = ANT_IDLE;
   }
   else
   {
      switch (state)
      {
      case ANT_IDLE:
         if (strncmp((const char *)buffer, "start", 5) == 0)
         {
            puts("Starting ANT with:");
            printf("\tChannel type: %s\n", (ant_setup.channel_type == PARAMETER_TX_NOT_RX) ? "master" : "slave");
            printf("\tPeriod: %d Hz\n", ant_setup.channel_period);
            printf("\tTx Power: %d\n", ant_setup.tx_power);
            printf("\tRadio freqs: %d %d %d\n", ant_setup.frequencies[0], ant_setup.frequencies[1], ant_setup.frequencies[2]);
            printf("\tFreq agility: %s\n", (ant_setup.frequency_agility_enabled == true) ? "enabled" : "disabled");

            if (ant_setup.frequency_agility_enabled)
            {
               ANT_AssignChannelExt(ANT_CHANNEL, ant_setup.channel_type, ANT_NET, 0x04);
            }
            else
            {
               ANT_AssignChannel(ANT_CHANNEL, ant_setup.channel_type, ANT_NET);
            }

            next_state = ANT_SETUP_SET_CHANNEL_ID;
         }
         break;

      case ANT_SETUP_SET_CHANNEL_ID:
         ANT_SetChannelId(ANT_CHANNEL, ANT_DEVICE_NO, ANT_DEVICE_TYPE, ANT_MANUFACTURE_ID);
         next_state = ANT_SETUP_SET_CHANNEL_PERIOD;
         break;

      case ANT_SETUP_SET_CHANNEL_PERIOD:
         ANT_SetChannelPeriod(ANT_CHANNEL, CLOCK_FREQUENCY / ant_setup.channel_period);
         next_state = ANT_SETUP_SET_CHANNEL_SEARCH_TIMEOUT;
         break;

      case ANT_SETUP_SET_CHANNEL_SEARCH_TIMEOUT:
         if (!lp_search_timeout_set)
         {
            ANT_SetChannelLowPrioritySearchTimeout(ANT_CHANNEL, 0);
            lp_search_timeout_set = true;
            next_state            = ANT_SETUP_SET_CHANNEL_SEARCH_TIMEOUT;
         }
         else
         {
            ANT_SetChannelSearchTimeout(ANT_CHANNEL, ant_setup.search_timeout);
            lp_search_timeout_set = false;
            next_state            = ANT_SETUP_SET_TX_POWER;
         }
         break;

      case ANT_SETUP_SET_TX_POWER:
         ANT_SetTransmitPower(ant_setup.tx_power);
         next_state = ANT_SETUP_SET_RADIO_FREQUENCY;
         break;

      case ANT_SETUP_SET_RADIO_FREQUENCY:
         if (ant_setup.frequency_agility_enabled)
         {
            ANT_SetFrequencyAgilitySettings(ANT_CHANNEL, &ant_setup.frequencies[0]);
         }
         else
         {
            ANT_SetChannelRFFreq(ANT_CHANNEL, ant_setup.frequencies[0]);
         }
         next_state = ANT_SETUP_OPEN_CHANNEL;
         break;

      case ANT_SETUP_OPEN_CHANNEL:
         ANT_OpenChannel(ANT_CHANNEL);
         next_state = ANT_RUN;
         break;

      case ANT_RUN:
         next_state = ANT_RUN;
         break;

      case ANT_COMPLETE:
         puts("\n================================================\nANT test results:");
         printf("\tSent packets: %d\n", sent_packets);
         printf("\tReceived packets: %d\n", received_packets);
         printf("\tLost packets: %d\n", lost_packets);
         printf("\tRetransmitted rx packets: %d\n", retransmitted_rx_packets);
         printf("\tRetransmitted tx packets: %d\n", retransmitted_tx_packets);
         printf("\tPackets with invalid counter: %d\n", invalid_counter_count);
         printf("\tDiscovery time: %d\n", discovery_time);
         puts("================================================\n");

         counter                  = 0;
         received_packets         = 0;
         lost_packets             = 0;
         sent_packets             = 0;
         retransmitted_tx_packets = 0;
         retransmitted_rx_packets = 0;
         invalid_counter_count    = 0;
         received_first_packet    = false;
         discovery_time           = 0;
         state = ANT_IDLE;
         break;

      case ANT_FAILURE:
         puts("ANT failed\n");
         break;

      default:
         printf("ant invalid state: %d\n", state);
         break;
      }
   }
}


void on_ant_cmd_response_event(uint8_t size, uint8_t *buffer)
{
   if (buffer[2] != RESPONSE_NO_ERROR)
   {
      printf("ANT cmd 0x%X failed with code 0x%X. Timestamp: %d ms\n", buffer[1], buffer[2], timer32_get_time());
      //next_state = ANT_FAILURE;
   }

   switch (buffer[1])
   {
   case MESG_UNASSIGN_CHANNEL_ID:
      next_state = ANT_COMPLETE;
      break;

   default:
      break;
   }
   state = next_state;
   lib_dispatcher_post_msg(HANDLE_PROCESS_ANT_COUNTER_APP, NULL, 0, NORMAL_PRIORITY);

}


void on_ant_event_tx(uint8_t size, uint8_t *buffer)
{

   static uint8_t toggle = 0;

   HAL_IO_SET_STATE(HAL_IO_LED2, toggle);
   toggle = !toggle;

   if (ant_setup.frequency_agility_enabled == false)
   {
      sent_packets++;
      send_buffer[0] = counter++;
      ANT_SendBroadcastData(ANT_CHANNEL, &send_buffer[0]);
   }
}


void on_ant_event_transfer_tx_completed(uint8_t size, uint8_t *buffer)
{
   static uint8_t toggle = 0;

   HAL_IO_SET_STATE(HAL_IO_LED3, toggle);
   toggle         = !toggle;
   send_buffer[0] = counter++;
   ANT_SendAcknowledgedData(ANT_CHANNEL, &send_buffer[0]);
   sent_packets++;
}


void on_ant_event_transfer_tx_failed(uint8_t size, uint8_t *buffer)
{
   retransmitted_tx_packets++;
}


void on_ant_event_rx_broadcast(uint8_t size, uint8_t *buffer)
{
   // When not using freq agility, this will process received data
   uint8_t counter_diff = buffer[1] - counter;

   if (!received_first_packet)
   {
      discovery_time = timer32_get_time();
   }

   if ((counter_diff == 0) && received_first_packet)
   {
      retransmitted_rx_packets++;
   }
   else if (counter_diff == 1)
   {
      received_packets++;
   }
   else
   {
      invalid_counter_count++;
   }
   counter = buffer[1];
   received_first_packet = true;
}


void on_ant_event_rx_acknowledged(uint8_t size, uint8_t *buffer)
{
   // When using freq agility, this method will process data
   uint8_t counter_diff = buffer[1] - counter;

   if (!received_first_packet)
   {
      discovery_time = timer32_get_time();
   }

   if ((counter_diff == 0) && received_first_packet)
   {
      retransmitted_rx_packets++;
   }
   else if (counter_diff == 1)
   {
      received_packets++;
   }
   else
   {
      invalid_counter_count++;
   }
   counter = buffer[1];
   received_first_packet = true;
}


void on_ant_event_rx_fail(uint8_t size, uint8_t *buffer)
{
   printf("ANT rx fail at %d ms\n", timer32_get_time());
   lost_packets++;
}


void on_ant_event_search_timeout(uint8_t size, uint8_t *buffer)
{
   puts("ANT rx search timeout\n");
   // Channel is closed automatically
}


void on_ant_event_channel_closed(uint8_t size, uint8_t *buffer)
{
   if (test_timed_out || !ant_setup.frequency_agility_enabled)
   {
      puts("ANT channel closed. Unassigning channel..\n");
      test_timed_out = false;
      ANT_UnAssignChannel(ANT_CHANNEL);
   }
   else
   {
      // When using freq agility and the node is the slave node, channel closed events
      // will occur when the frequency is changed.
      puts("ANT changed frequency\n");
   }
}


void on_ant_unexpected_event(uint8_t size, uint8_t *buffer)
{
   printf("ANT unknown event: 0x%X 0x%X\n", buffer[1], buffer[2]);
}


void on_ant_mesg_invalid_id(uint8_t size, uint8_t *buffer)
{
   uint8_t i;

   puts("ANT Invalid id: ");
   for (i = 0; i < size; i++)
   {
      printf("0x%X ", buffer[i]);
   }
   puts("\n");
}


void on_ant_system_startup(uint8_t size, uint8_t *buffer)
{
   puts("ANT device started.\n");
   counter                  = 0;
   received_packets         = 0;
   lost_packets             = 0;
   sent_packets             = 0;
   retransmitted_tx_packets = 0;
   retransmitted_rx_packets = 0;
   received_first_packet    = false;
   discovery_time           = 0;
   state      = ANT_IDLE;
   next_state = ANT_IDLE;
   //lib_dispatcher_post_msg(HANDLE_PROCESS_ANT_COUNTER_APP, NULL, 0, NORMAL_PRIORITY);
}


void on_ant_timer_interrupt(uint8_t size, uint8_t *buffer)
{
   if (state == ANT_RUN)
   {
      test_timed_out = true;
      ANT_CloseChannel(ANT_CHANNEL);
   }
}
