/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 4808 $
 */ 

/** @file
@brief Implementation of the ACI transport layer module
*/
#include <stdio.h>
#include "hal/hal_aci_tl.h"
#include "hal/hal_io.h"
#include "LPC11xx.h"
#include <stdint.h>
#include "bool.h"
//#include <intrins.h>
#include "system.h"
#include "lib/assertions.h"

static hal_aci_data_t received_data;
static hal_aci_data_t default_data;
static volatile hal_aci_data_t *p_data_to_send;
static volatile bool spi_transaction_running;
static volatile bool spi_transmit_requested;

static uint8_t spi_readwrite(uint8_t aci_byte); 

#define spi_enable() (LPC_SSP0->CR1 |= 0x02)
#define spi_disable() (LPC_SSP0->CR1 &= ~0x02)


void hal_aci_tl_disable_transaction(void)
{
	GPIOIntDisable( PORT2, 9 );
 // INTEXP &= ~0x10; // Disable GPINT1 (P0.6)
}

void hal_aci_tl_enable_transaction(void)
{
  //INTEXP |= 0x10; // Enable GPINT1 (P0.6)
	GPIOIntEnable( PORT2, 9 );
}

void hal_aci_tl_init()
{
  received_data.buffer[0] = 0;
  GPIOInit();
  LPC_SYSCON->PRESETCTRL |= (0x1<<0);
 	  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<11);
 	  LPC_SYSCON-> SSP0CLKDIV = 0x02;			/* Divided by 2 */
 	// TODO: insert code here
 	  LPC_IOCON->PIO0_8           &= ~0x07;	/*  SSP I/O config */
 	  LPC_IOCON->PIO0_8           |= 0x01;		/* SSP MISO */
 	  LPC_IOCON->PIO0_9           &= ~0x07;
 	  LPC_IOCON->PIO0_9           |= 0x01;		/* SSP MOSI */


 	  /* On HummingBird 1(HB1), SSP CLK can be routed to different pins,
 	  other than JTAG TCK, it's either P2.11 func. 1 or P0.6 func. 2. */
 	  LPC_IOCON->SCK_LOC = 0x01;
 	  LPC_IOCON->PIO2_11 = 0x01;/* P2.11 function 1 is SSP clock, need to combined
 		  						with IOCONSCKLOC register setting */





 	  GPIOSetDir( PORT1, 8, 1 ); //Reset
 	  GPIOSetValue( PORT2, 0, 1);


 	LPC_SSP0->CR0 |= 0x07; // 8 bit
 	 LPC_SSP0->CR0 |= 0x00; //CPHA = 0; //CPOL=0 //FRF=0(SPI)
 	 LPC_SSP0->CR0 |= 0x0F00; /// Divde SPI clock by 16
 	 LPC_SSP0->CPSR = 0x02; //SPI clock prescaler=2
 	 spi_enable();

 	//HAL_IO_CONFIG(HAL_IO_RADIO_CSN, HAL_IO_OUTPUT, HAL_IO_OUTPUT_NORMAL_STRENGTH);
 	 GPIOSetDir( PORT2, 10, 1 ); //CSN
 	  GPIOSetValue( PORT2, 10, 1 );
 	  //HAL_IO_CONFIG(HAL_IO_RADIO_ACTIVE, HAL_IO_INPUT, HAL_IO_INPUT_BUF_ON_NO_PULL);
 	 GPIOSetDir( PORT1, 9, 0 ); //Active

 	  //HAL_IO_CONFIG(HAL_IO_RADIO_REQN, HAL_IO_OUTPUT, HAL_IO_OUTPUT_NORMAL_STRENGTH);
 	 GPIOSetDir( PORT2, 8, 1 ); //REQN
 	 GPIOSetValue( PORT2, 8, 1 );
 	  //HAL_IO_CONFIG(HAL_IO_RADIO_RDY, HAL_IO_INPUT, HAL_IO_INPUT_BUF_ON_PULL_UP);
 	GPIOSetDir( PORT2, 9, 0 ); //RDY
 	  /* port2_9, single trigger, active falling edge. */
 	  GPIOSetInterrupt( PORT2, 9, 0, 0, 0 );
 	  GPIOIntEnable( PORT2, 9 );
  //WUPIN = 1;                                          //Enable wake-up from pin interrupt
  //WUIRQ = 1;

  hal_aci_tl_enable_transaction();
  //EX0 = 1; // Enable pin interrupt
  //IT0 = 1; // Falling edge

  HAL_IO_SET_STATE(HAL_IO_RADIO_REQN, 1);
  spi_transaction_running = false;
  spi_transmit_requested = false;
  p_data_to_send = &default_data;
  p_data_to_send->buffer[0] = 0;
}

bool hal_aci_tl_send(hal_aci_data_t *p_aci_cmd)
{
  bool previous_ea;

  ASSERT(ERROR_CODE_HAL_ACI_TL_OVERFLOW,(p_aci_cmd->buffer[0] <= HAL_ACI_MAX_LENGTH));
  if (p_aci_cmd->buffer[0] > HAL_ACI_MAX_LENGTH)
  {
    p_aci_cmd->buffer[0] = HAL_ACI_MAX_LENGTH;
  }

  if (spi_transmit_requested)
  {
    return(false);
  }

  previous_ea = ARE_INTERRUPTS_ENABLED();
  DISABLE_INTERRUPTS();                           /*disable interrupts to protects the modification of the buffer pointer*/
  p_data_to_send = p_aci_cmd;
  if (previous_ea)
  {
    ENABLE_INTERRUPTS();                         /*enventually re-enable the interrupts if they were enabled*/
  }

  // Request transmission
  spi_transmit_requested = true;
  HAL_IO_SET_STATE(HAL_IO_RADIO_REQN, 0);
  return(true);
}


void hal_aci_tl_on_ready_signal(void)
{
	  uint8_t byte_cnt;
	  uint8_t byte_sent_cnt;
	  uint8_t max_bytes;
	  //bool is_transmit_finished = false;


	  if(1 ==HAL_IO_READ(HAL_IO_RADIO_RDY))
	  {
	    return;
	  }

	  spi_transaction_running = true;
	  //SPIMCON0 |= SPI_ENABLE;
	  spi_enable();
	  HAL_IO_SET_STATE(HAL_IO_RADIO_REQN, 0);


	  // Send length, receive header
	  byte_sent_cnt = 0;
	  received_data.status_byte = spi_readwrite(p_data_to_send->buffer[byte_sent_cnt++]);
	  // Send first byte, receive length from slave
	  received_data.buffer[0] = spi_readwrite(p_data_to_send->buffer[byte_sent_cnt++]);
	  if (0 == p_data_to_send->buffer[0])
	  {
	    max_bytes = received_data.buffer[0];
	  }
	  else
	  {
	    // Set the maximum to the biggest size. One command byte is already sent
	    max_bytes = (received_data.buffer[0] > (p_data_to_send->buffer[0] - 1))
	      ? received_data.buffer[0] : (p_data_to_send->buffer[0] - 1);
	  }

	  if (max_bytes > HAL_ACI_MAX_LENGTH)
	  {
	    max_bytes = HAL_ACI_MAX_LENGTH;
	  }

	  // Transmit/receive the rest of the packet
	  for (byte_cnt = 0; byte_cnt < max_bytes; byte_cnt++)
	  {
	    received_data.buffer[byte_cnt+1] =  spi_readwrite(p_data_to_send->buffer[byte_sent_cnt++]);
	  }
	  /* Release REQN*/
	  HAL_IO_SET_STATE(HAL_IO_RADIO_REQN, 1);

	  //SPIMCON0 &= ~SPI_ENABLE;
	  spi_disable();

	  ASSERT(ERROR_CODE_HAL_ACI_TL_STATUS_BYTE,(0 != received_data.status_byte));

	  if (spi_transmit_requested)
	  {
	    p_data_to_send->buffer[0] = 0;
	    spi_transmit_requested = false;
	  }


	  spi_transaction_running = false;

	  /*
	  puts("sent: ");
	  uint8_t i;
	  for (i=0; i<byte_sent_cnt; i++)
		  printf("0x%X ", p_data_to_send->buffer[i]);
	  puts("\r\nreceived: ");
	  for (i=0; i<received_data.buffer[0]; i++)
		  printf("0x%X ", received_data.buffer[i]);
	  puts("\r\n");
		*/


	  /* valid Rx available or transmit finished*/
	  hal_aci_tl_msg_rcv_hook(&received_data);


}

static uint8_t spi_readwrite(const uint8_t aci_byte)
{
	//printf("sending byte %x\n", aci_byte);
	 const uint8_t lookup[] = { 0, 8,  4, 12, 2, 10, 6, 14,1, 9, 5, 13,3, 11, 7, 15 };
	 volatile uint8_t temp_val;
	 LPC_SSP0->DR=(uint16_t)(((lookup[(aci_byte & 0x0F)]) << 4) + lookup[((aci_byte & 0xF0) >> 4)]);
	 while(((LPC_SSP0->SR) & 0x10))
		 ;
	 temp_val= (uint8_t)(LPC_SSP0->DR);
	 //printf("received byte %x\n", temp_val);

	 return (uint8_t)(((lookup[(temp_val & 0x0F)]) << 4) + lookup[((temp_val & 0xF0) >> 4)]);
}


