/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */ 

/** @file
 * @brief messages dispatcher module
 */
 
/** @defgroup lib_dispatcher lib_dispatcher
* @{
* @ingroup lib
*  
* @brief Message dispatcher module
* 
* The message dispatcher module allows an application to send messages and to get
* a notification when a message is dispatched. A message can be sent with 3 different level of priority.
* 
* For this module you can set two compile option define : DISPATCHER_WITH_SWITCH_CASE and POST_WITH_DATA.
* If you set DISPATCHER_WITH_SWITCH_CASE, this module use a switch/case implementation, otherwise, it uses functions pointers.
* If you set POST_WITH_DATA, then the post functionality will take a pointer and a size which will be passed to each subscriber function.
*
* To facilitate the use of this message dispatcher, you can use the Dispatcher Tool
* in nRFgo Studio.
* In this tool, you can define handles (right click and 'add_handle'). The handles can
* have any name and up to 3 subscribers attach to it.
* 
* The subscribers are functions that will get called when the corresponding handle will
* be triggered. You have the possibility to have data attached to theses function by defining POST_WITH_DATA.
* In this case the functions will get a pointer and a size as parameter and the lib_dispatcher_post_msg will
* have those same parameters in its prototype.
* 
* The dispatcher must be initialised by a call to lib_dispatcher_init() and then the function
* lib_dispatcher_dispatch should be called in an infinite loop, as shown in the following example.
* @code
* void main(void)
* {
*   lib_dispatcher_init();
*   ENABLE_INTERRUPTS();
*   for (;;)
*   {
*     lib_dispatcher_dispatch();
*   }
* }
* @endcode
* 
* <h4>Requirements</h4>
* <h3>HAL</h3>
* This library uses functions from hal_power module for handling power management during idle periods:
*  - hal_power_sleep() which put the device in register_retention sleep, with timer if the timer has been started;
* <h3>LIB</h3>
* This library uses the @ref lib_traces library for @ref LIB_TRACES_LOG_INFO and @ref LIB_TRACES_LOG_ERROR macros as well as the @ref lib_assert library for @ref ASSERT macros.
* 
*/
 #ifndef DISPATCHER_H__
 #define DISPATCHER_H__

#include "bool.h"
#include <stdint.h>

#include "dispatcher_config.h"
#include "system.h"

#define NB_MAX_EVT_PER_PRIORITY 50



typedef struct lib_dispatcher_data_t{
  uint8_t handle;
#ifdef POST_WITH_DATA
  uint8_t data_size;
  void *p_data;
#endif //POST_WITH_DATA
}lib_dispatcher_data_t;

/** @brief Callback on event
 *  @details The functions to get an event notification has to have this prototype.
 *  @param size : number of bytes associated with the message
 *  @param p_msg_data : pointer to the data associated with the message
 */
#ifdef POST_WITH_DATA
typedef void (*dispatcher_cb_t)(uint8_t size, uint8_t *p_msg_data); // removed "code"
#else
typedef void (*dispatcher_cb_t)(); // removed "code"
#endif //POST_WITH_DATA

/** @brief Message priorities of posted messages */
typedef enum lib_dispatcher_priority_t{
  HIGH_PRIORITY      = 0,
  NORMAL_PRIORITY,
  LOW_PRIORITY,
  NB_PRIORITIES
}lib_dispatcher_priority_t;

#ifdef POST_WITH_DATA
/** @brief Function to post a immediate message
 *  @details Use this function to post a message without any delay. This message will be dispatched to all subscribers as soon as possible.
 *  @param msg_handle Message handle
 *  @param p_buffer Data to associate with the message (if POST_WITH_DATA is defined)
 *  @param size Size of data (if POST_WITH_DATA is defined)
 *  @param msg_priority Message priority.
 */
void lib_dispatcher_post_msg(uint8_t msg_handle, uint8_t *p_buffer, uint8_t size, lib_dispatcher_priority_t msg_priority); // removed "reentrant"
#else

/** @brief Function to post a immediate message
 *  @details Use this function to post a message without any delay. This message will be dispatched to all subscribers as soon as possible.
 *  @param msg_handle Message handle
 *  @param msg_priority Message priority.
 */
void lib_dispatcher_post_msg(uint8_t msg_handle, lib_dispatcher_priority_t msg_priority); // removed reentrant
#endif //POST_WITH_DATA

/** @brief Main dispatcher function
 *  @details This function should be called in a permanent loop. It handles the dispatching of all pending messages.
 * When no more messages are pending, it goes in register retention sleep mode (@ref hal_power).
 */
void lib_dispatcher_dispatch(void); // removed reentrant

/** Function to be initialize the dispatcher*/
void lib_dispatcher_init();

#endif // DISPATCHER_H__
/** @} */
