/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */ 

/** @file
 * @brief Interface for hal_aci_tl.
 */
 
/** @defgroup hal_aci_tl hal_aci_tl
@{
@ingroup hal
 
@brief Module for the ACI Transport Layer interface
@details This module is responsible for the hardware specific interface to 
nRF8001 using ACI. The hal_aci_tl_send_cmd() can be called directly to send ACI commands.


hal_aci_rcv_event() should be called from the RDY signal's ISR. The function
will receive the event and call the receive the hook hal_aci_tl_msg_rcv_hook().

*/
 
#ifndef HAL_ACI_TL_H__
#define HAL_ACI_TL_H__

#include "bool.h"
#include <stdint.h>

#ifndef HAL_ACI_MAX_LENGTH
#define HAL_ACI_MAX_LENGTH 31
#endif //HAL_ACI_MAX_LENGTH


/** Data type for ACI commands and events */
typedef struct hal_aci_data_t{
  uint8_t status_byte;
  uint8_t buffer[HAL_ACI_MAX_LENGTH+1];
} hal_aci_data_t;


/** @brief Message received hook function
 *  @details A hook function that must be implemented by the client of this module. 
 * The function will be called by hal_aci when a new message has been received from the radio.
 *  @param received_msg pointer to a structure containing the number of received bytes and a pointer to the received data
 */
extern void hal_aci_tl_msg_rcv_hook(hal_aci_data_t *received_msg);

/** ACI Transport Layer configure inputs/outputs
 */
void hal_aci_tl_io_config();


/** ACI Transport Layer initialisation
 */
void hal_aci_tl_init();

/**@brief Sends an ACI command to the radio
 *  @details
 *  This function sends an ACI command to the radio. This will memorize the pointer of message to send and 
 *  lower the request line. When the device will in turn lower the ready line, the correponding interrupt 
 *  will do the actual sending of data.
 *  @param aci_buffer Pointer to the message to send
 *  @return True if the send is started successfully, false if a transaction is already running.
 */
bool hal_aci_tl_send(hal_aci_data_t *aci_buffer);

/** @brief Receives an ACI event from the radio
 *  @details
 *  This function must be called when RDY goes low (active), for example in the
 *  ISR. The function will then receive the event over SPI and
 *  call the event callback function defined in hal_aci_tl_init().
 *  If the send_cmd has been called previously, it will also send the message to the device.
 */
void hal_aci_tl_on_ready_signal(void);


/** @brief function to disable transaction
 *  @details
 *  This function disable the ready line interrupt thus avoiding initiating any transaction on the ACI.
 */
void hal_aci_tl_disable_transaction(void) ;

/** @brief function to enable transaction
 *  @details
 *  This function enable the ready line interrupt.
 */
void hal_aci_tl_enable_transaction(void) ;

#endif // HAL_ACI_TL_H__
/** @} */




