/*****************************************************************************
 *   uart.c:  UART API file for NXP LPC11xx Family Microprocessors
 *
 *   Copyright(C) 2008, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2009.12.07  ver 1.00    Preliminary version, first Release
 *
******************************************************************************/

#include "uart.h"

extern void ANT_SerialHaveMessage(uint8_t* msg);

volatile uint32_t UARTStatus;
volatile uint8_t  UARTTxEmpty = 1;
static uint8_t  UARTBuffer[BUFSIZE];
static uint32_t UARTCount = 0;


/*****************************************************************************
** Function name:		UART_IRQHandler
**
** Descriptions:		UART interrupt handler
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void UART_IRQHandler(void)
{
  uint8_t IIRValue, LSRValue;
  uint8_t Dummy = Dummy;
  static uint8_t size = 0;

  IIRValue = LPC_UART->IIR;
    
  IIRValue >>= 1;			/* skip pending bit in IIR */
  IIRValue &= 0x07;			/* check bit 1~3, interrupt identification */

  if (IIRValue == IIR_RDA)	/* Receive Data Available */
  {
    /* Receive Data Available */
    UARTBuffer[UARTCount++] = LPC_UART->RBR;
    if (UARTCount == BUFSIZE)
    {
      UARTCount = 0;		/* buffer overflow */
      return;
    }

  }


  if (UARTBuffer[0] != 0xA4 && UARTBuffer[0] != 0xA5) { // 0xA4/A5 is the synch byte from ANT
	  UARTCount = 0; // ignore
	  size = 0;
	  return;
  }
  else {

	  if (UARTCount == 2) {
		  size = UARTBuffer[1];
	  }
	  else if (size > 0) {
		  if (UARTCount > (size+4)) {
			  // Errors have occurred, ignore message
			  UARTStatus++;
			  size = 0;
			  UARTCount = 0;
		  }
		  else if (UARTCount == (size+4)) { // just received the checksum byte
			  LPC_UART->IER = 0; // disable UART interrupts
			  size = 0;
			  UARTCount = 0;
			  return ANT_SerialHaveMessage(&UARTBuffer[0]);
		  }
	  }
  }

  //ANT_SerialHaveRawData(UARTBuffer[UARTCount-1]);
  return;
}


/*****************************************************************************
** Function name:		UARTInit
**
** Descriptions:		Initialize UART0 port, setup pin select,
**				clock, parity, stop bits, FIFO, etc.
**
** parameters:			UART baudrate
** Returned value:		None
** 
*****************************************************************************/
void UARTInit(uint32_t baudrate)
{
  uint32_t Fdiv;
  uint32_t regVal;

  UARTTxEmpty = 1;
  UARTCount = 0;
  
  NVIC_DisableIRQ(UART_IRQn);

  LPC_IOCON->PIO1_6 &= ~0x07;    /*  UART I/O config */
  LPC_IOCON->PIO1_6 |= 0x01;     /* UART RXD */
  LPC_IOCON->PIO1_7 &= ~0x07;	
  LPC_IOCON->PIO1_7 |= 0x01;     /* UART TXD */
  LPC_IOCON->PIO0_7 &= ~0x07;
  LPC_IOCON->PIO0_7 |= 0x01; 	/* Select function !CTS */

  /* Enable UART clock */
  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12);
  LPC_SYSCON->UARTCLKDIV = 0x1;     /* divided by 1 */

  LPC_UART->LCR = 0x83;             /* 8 bits, no Parity, 1 Stop bit */
  regVal = LPC_SYSCON->UARTCLKDIV;

  Fdiv = (((SystemCoreClock*LPC_SYSCON->SYSAHBCLKDIV)/regVal)/16)/baudrate ;	/*baud rate */

  LPC_UART->DLM = Fdiv / 256;							
  LPC_UART->DLL = Fdiv % 256;
  LPC_UART->LCR = 0x03;		/* DLAB = 0 */
  LPC_UART->FCR = 0x07;		/* Enable and reset TX and RX FIFO. */
  LPC_UART->MCR |= 0x80; 	/* Enable Auto-CTS */

  /* Read to clear the line status. */
  regVal = LPC_UART->LSR;

  /* Ensure a clean start, no data in either TX or RX FIFO. */
// CodeRed - added parentheses around comparison in operand of &
  while (( LPC_UART->LSR & (LSR_THRE|LSR_TEMT)) != (LSR_THRE|LSR_TEMT) );
  while ( LPC_UART->LSR & LSR_RDR )
  {
	regVal = LPC_UART->RBR;	/* Dump data from RX FIFO */
  }
 
  /* Enable the UART Interrupt */
  NVIC_EnableIRQ(UART_IRQn);
  LPC_UART->IER = IER_RBR; // | IER_RLS;	/* Enable UART interrupt */

  return;
}

/*****************************************************************************
** Function name:		UARTSend
**
** Descriptions:		Send a block of data to the UART 0 port based
**				on the data length
**
** parameters:		buffer pointer, and data length
** Returned value:	None
** 
*****************************************************************************/
void UARTSend(uint8_t *BufferPtr, uint32_t Length)
{
  
  while ( Length != 0 )
  {
	  while ( !(LPC_UART->LSR & LSR_THRE) );
	  LPC_UART->THR = *BufferPtr;
      BufferPtr++;
      Length--;
  }
  return;
}


/******************************************************************************
**                            End Of File
******************************************************************************/
