/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */
#ifndef LIB_ACI_H__
#define LIB_ACI_H__

/** @file
* @brief ACI library
*/

/** @defgroup lib_aci lib_aci
@{
@ingroup lib

@brief Library for the logical part of the Application Controller Interface (ACI)

@details This library implements functions for the ACI commands, and handles ACI
events. See @ref p_lib_aci for information on how to use this library.

*/


#include "hal_aci_tl.h"
#include "aci.h"
#include "aci_cmds.h"
#include "aci_evts.h"

#include <stdint.h>


#define ERROR_CMD_RESPONSE_ERROR 0xb2
#define ERROR_RADIO_EVT_ERROR 0xc3
#define ERROR_RADIO_EVT_UNKNOWN 0xd4
#define ERROR_CMD_RESPONSE_WRONG_CMD 0xd5
#define ERROR_CMD_RESPONSE_TOO_SHORT 0xd8

#define LIB_ACI_SIZE_FIFO_MSG 1

#define EVT_CMD_RESPONSE_MIN_LENGTH              3

typedef struct lib_aci_error_descriptor_t {
  uint8_t error_code;
  uint8_t error_sub_code1;
  uint8_t error_sub_code2;
} lib_aci_error_descriptor_t;


#define DISCONNECT_REASON_CX_TIMEOUT                 0x08
#define DISCONNECT_REASON_CX_CLOSED_BY_PEER_DEVICE   0x13
#define DISCONNECT_REASON_POWER_LOSS                 0x14
#define DISCONNECT_REASON_CX_CLOSED_BY_LOCAL_DEVICE  0x15
#define DISCONNECT_REASON_ADVERTISER_TIMEOUT         0x50


/** @brief Hook function for receiving messages from the ACI library.
 *  @details A hook function that must be implemented by the client of this module.
 *  The function will be called each time an event is received from the radio or an error occurred.
 *  @param identifier The cause for calling the function.
 */
extern void lib_aci_post_msg_hook(uint8_t identifier);

/** @name Functions for library management */
//@{

/** @brief Initialization function.
 */
void lib_aci_init();

/** @brief Checks if the host setup is complete.
 *  @return True if the setup is complete, false if the status is continue.
 */
bool lib_aci_is_host_setup_complete();

/** @brief Gets the number of currently available ACI credits.
 *  @return Number of ACI credits.
 */
uint8_t lib_aci_get_nb_available_credits();

/** @brief Gets the connection interval in milliseconds.
 *  @return Connection interval in milliseconds.
 */
uint16_t lib_aci_get_cx_interval_ms();

/** @brief Checks if a given pipe is available.
 *  @param pipe Pipe to check.
 *  @return True if the pipe is available, otherwise false.
 */
bool lib_aci_is_pipe_available(uint8_t pipe);

/** @brief Checks if a given pipe is closed.
 *  @param pipe Pipe to check.
 *  @return True if the pipe is closed, otherwise false.
 */
bool lib_aci_is_pipe_closed(uint8_t pipe);

/** @brief Check if the discovery operation is finished
 *  @return True if the the discovery is finished
 */
bool lib_aci_is_discovery_finished();

/** @brief Enables a new transaction on the ACI after the reception of an event.
 *  @details When the ACI library has received an ACI event, all transactions
 *  are blocked until this function has been called. Normally an interpreter
 *  function is called when receiving an event. When the event is interpreted
 *  call this function.
 */
void lib_aci_enable_transmission(void);

/** @brief Interprets the latest received event as a aci_evt_params_cmd_rsp_t structure.
 *  @details If the latest received event is a @c CommandResponseEvent, call
 *  this function to interpret it as an aci_evt_params_cmd_rsp_t structure.
 *  @param[out] p_out_data Pointer to an aci_evt_params_cmd_rsp_t structure.
 */
void lib_aci_interpret_evt_cmd_rsp(aci_evt_params_cmd_rsp_t *p_out_data);

/** @brief Interpret the latest received event as an aci_evt_params_device_started_t structure.
 *  @details If the latest received event is a @c DeviceStartedEvent, call this
 *  function to interpret it as an aci_evt_params_device_started_t structure.
 *  @param[out] p_out_data Pointer to an aci_evt_params_device_started_t structure.
 */
void lib_aci_interpret_evt_device_started(aci_evt_params_device_started_t *p_out_data);

/** @brief Interprets the latest received event as an aci_evt_params_pipe_status_t structure.
 *  @details If the latest received event is a @c PipesConnectedEvent, call
 *  this function to interpret it as an aci_evt_params_pipe_status_t structure.
 *  @param[out] p_out_data Pointer to an aci_evt_params_pipe_status_t structure.
 */
void lib_aci_interpret_evt_pipes_status(aci_evt_params_pipe_status_t *p_out_data);

/** @brief Interpret the latest received event as an aci_evt_params_disconnected_t structure.
 *  @details If the latest received event is a @c DisconnectedEvent, call
 *  this function to interpret it as an aci_evt_params_disconnected_t
 *  structure.
 *  @param[out] p_out_data Pointer to an aci_evt_params_disconnected_t structure.
 */
void lib_aci_interpret_evt_disconnected(aci_evt_params_disconnected_t *p_out_data);

/** @brief Interpret the latest received event as an aci_evt_params_data_received_t structure.
 *  @details If the latest received event is a @c DataReceivedEvent, call
 *  this function to interpret it as an aci_evt_params_data_received_t
 *  structure.
 *  @param[out] p_out_data pointer to an aci_evt_params_data_received_t structure.
 *  @return Length of the received data
 */
uint8_t lib_aci_interpret_evt_data_rcvd(aci_evt_params_data_received_t *p_out_data);

/** @brief Interpret the latest received event as a aci_evt_params_hw_error_t structure.
 *  @details If the latest received event is a @c HardwareErrorEvent, call this
 *  function to interpret it as an aci_evt_params_hw_error_t structure.
 *  @param[out] p_out_data Pointer to an aci_evt_params_hw_error_t structure.
 *  @return Length of name_of_file member in p_out_data
 */
uint8_t lib_aci_interpret_evt_hw_error(aci_evt_params_hw_error_t *p_out_data);

/** @brief Interpret the latest received event as an aci_evt_params_data_credit_t structure.
 *  @details If the latest received event is a @c CreditEvent, call this
 *  function to interpret it as an aci_evt_params_data_credit_t structure.
 *  @return Number of received credits.
 */
uint8_t lib_aci_interpret_evt_credit();

/** @brief Interpret the latest received event as an aci_evt_params_timing_t structure.
 *  @details If the latest received event is a @c TimingEvent, call this
 *  function to interpret it as an aci_evt_params_timing_t structure.
 *  @param[out] p_out_data Pointer to an aci_evt_params_timing_t structure.
 */
void lib_aci_interpret_evt_timing(aci_evt_params_timing_t *p_out_data);


/** @brief Interpret the latest received event as an aci_evt_params_pipe_error_t structure.
 *  @details If the latest received event is a @c PipeErrorEvent, call this
 *  function to interpret it as an aci_evt_params_pipe_error_t structure.
 *  @return Pipe number on which the error occurred.
 */
uint8_t lib_aci_interpret_evt_pipe_error(aci_evt_params_pipe_error_t *p_out_data);

/** @brief Retrieves the details associated to an error
 *  @details If the lib_aci post an HANDLE_EVT_ERROR, the subscriber function
 *  can use this to get more information about the error.
 *  @param[out] p_error_data Pointer to a lib_aci_error_descriptor_t structure.
 */
void lib_aci_interpret_error(lib_aci_error_descriptor_t *p_error_data);

/** @brief Interpret the latest received event as an echo.
 *  @details If the latest received event is an @c EchoEvent, call this
 *  function to get its actual content.
 *  @param[out] p_out_data Pointer to a buffer which this function will write the received data.
 *  @return Length of the received data.
 */
uint8_t lib_aci_interpret_evt_echo(uint8_t *p_out_data);

/** @brief Interprets the latest received event as an aci_evt_params_connected_t structure.
 *  @details If the latest received event is a @c ConnectedEvent, call this
 *  function to interpret it as an aci_evt_params_connected_t structure.
 *  @param[out] p_out_data Pointer to an aci_evt_params_connected_t  structure.
 */
void lib_aci_interpret_evt_connected(aci_evt_params_connected_t *p_out_data);

/** @brief Interpret the latest received event as an aci_evt_params_bond_status_t structure.
 *  @details If the latest received event is a @c BondStatusEvent, call this
 *  function to interpret it as an aci_evt_params_bond_status_t structure.
 *  @param[out] p_out_data Pointer to an aci_evt_params_bond_status_t structure.
 */
void lib_aci_interpret_evt_bond_status(aci_evt_params_bond_status_t *p_out_data);

//@}

/** @name ACI Commands available in all modes */
//@{

/** @brief Sets the radio in Sleep mode
 *  @details The function sends a @c Sleep command to the radio. The radio is
 *  in Sleep mode when lib_aci posts a HANDLE_TRANSACTION_FINISHED handle.
 *  @return True if the command was transaction has been successfully initiated
 */
bool lib_aci_sleep();

/** @brief Gets the radio's version.
 *  @details This function sends a @c GetDeviceVersion command. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the version is received.
 *  @return True if the transaction has been successfully initiated.
 */
bool lib_aci_device_version();

/** @brief Gets the device address.
 *  @details This function sends a @c GetDeviceAddress command. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the device address is received.
 *  @return true if the transaction has been successfully initiated
 */
bool lib_aci_get_address();

//@}

/** @name ACI commands available in Sleep mode */
//@{

/** @brief Wakes up the radio
 *  @details This function sends a @c Wakeup command to wake up the radio from
 *  Sleep mode. When woken up the radio will send a @c DeviceStartedEvent and
 *  a @c CommandResponseEvent. lib_aci will post messages for these two events,
 *  and a HANDLE_TRANSACTION_FINISHED.
 *  @return True if the transaction has been successfully initiated
 */
bool lib_aci_device_wakeup();

//@}

/** @name ACI commands available in Active mode */
//@{

/** @brief Sets the radio in Test mode
 *  @details This function sends a @c Test command to the radio. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the command is sent. There are two
 *  Test modes available:
 *  - UART: DTM commands are received over UART.
 *  - ACI: DTM commands are received over ACI.
 *  The same command is used to exit the Test mode When receiving
 *  a @c DeviceStartedEvent the radio has entered the new mode.
 *  @param enter_exit_test_mode Enter a Test mode, or exit Test mode.
 *  @return True if the transaction has been successfully initiated
 */
bool lib_aci_set_test_mode(aci_test_mode_change_t enter_exit_test_mode);

/** @brief Set the radio's TX power.
 *  @details This function sends a @c SetTxPower command. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the @c CommandResponseEvent has been received.
 *  @param tx_power TX power to be used by the radio.
 *  @return True if the transaction has been successfully initiated
 */
bool lib_aci_set_radio_tx_power(aci_device_output_power_t tx_power);

/** @brief Tries to connect to a peer device.
 *  @details This function sends a @c Connect command to the radio. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the @c CommandResponseEvent has been
 *  received.
 *  @param run_timeout Maximum advertising time in seconds (0 means infinite).
 *  @param adv_interval Advertising interval (in multiple of 0.625&nbsp;ms)
 *  @return True if the transaction has been successfully initiated
 */
bool lib_aci_connect(uint16_t run_timeout, uint16_t adv_interval);

/** @brief Tries to bond with a peer device.
 *  @details This function sends a @c Bond command to the radio. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the @c CommandResponseEvent has been
 *  received.
 *  @param run_timeout Maximum advertising time in seconds (0 means infinite).
 *  @param adv_interval Advertising interval (in multiple of 0.625&nbsp;ms)
 *  @return True if the transaction has been successfully initiated
 */
bool lib_aci_bond(uint16_t run_timeout, uint16_t adv_interval);

/** @brief disconnect from peer device
 *  @details This function sends a @c Disconnect command to the radio. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the @c CommandResponseEvent has been
 *  received.
 *  @param reason reason for disconnecting.
 *  @return True if the transaction has been successfully initiated
 */
bool lib_aci_disconnect(aci_disconnect_reason_t reason);

/**@brief Set Local Data
 *  @details
 *  This command updates the value of characteristic value or characteristic descriptaro that are stored locally on the device
 *  Could could be called for all types of pipes as long as the data is stored locally
 *
 *  @param pipe pipe number on which the data should be set
 *  @param value pointer to the data to set.
 *  @param size size of the data to set.
 *  @return true if the transaction has been successfully initiated
*/
bool lib_aci_set_local_data(uint8_t pipe, uint8_t *value, uint8_t size);

//@}

/** @name ACI commands available in Connected mode */
//@{


/** @brief Opens a remote pipe
 *  @details This function sends an @c OpenRemotePipe command. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the @c CommandResponseEvent has been
 *  received.
 *  @param pipe Number of the pipe to open.
 *  @return True if the transaction has been successfully initiated
 */
bool lib_aci_open_remote_pipe(uint8_t pipe);

/** @brief Sends data on a given pipe.
 *  @details This function sends a @c SendData command with application data to
 *  the radio. lib_aci will post a HANDLE_TRANSACTION_FINISHED when the command
 *  has been sent. This function will memorize use of credits, and check that
 *  enough credits are available.
 *  @param pipe Pipe number on which the data should be sent
 *  @param value Pointer to the data to send.
 *  @param size Size of the data to send.
 *  @return True if the transaction has been successfully initiated.
 */
bool lib_aci_send_data(uint8_t pipe, uint8_t *value, uint8_t size);

/** @brief Request data from a given pipe.
 *  @details This function sends a @c RequestData command to the radio. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the command has been sent. This
 *  function will memorize use of credits, and check that enough credits are available.
 *  After this command, the radio should send back either a @c DataReceivedEvent
 *  or a @c PipeErrorEvent.
 *  @param pipe Pipe number on which the data is requested.
 *  @return True if the transaction has been successfully initiated.
 */
bool lib_aci_request_data(uint8_t pipe);

//@}

/** @name ACI commands available in Setup mode */
//@{

/** @brief Send the generated configuration messages
 *  @details This function sends a generated configuration message. The messages
 *  are defined in services.h (one for each project) which is included by
 *  lib_aci.c. lib_aci will post a HANDLE_TRANSACTION_FINISHED when the the
 *  @c CommandResponseEvent is received. This function must be called until it
 *  returns true. Once all this messages has been sent, the radio goes
 *  automatically in Active mode.
 *  @return State of the Setup procedure
 *  @retval true The setup is complete
 *  @retval false Call the function again to continue.
 */
bool lib_aci_send_setup_msg();


/** @name ACI commands available in Test mode */
//@{

/** @brief Send an echo message
 *  @details This function sends an @c Echo command to the radio. lib_aci will
 *  post a HANDLE_TRANSACTION_FINISHED when the command has been sent.
 *  @param message_size Length of the data to send.
 *  @param message_data Pointer to the data to send.
 *  @return True if the transaction has been successfully initiated.
*/
bool lib_aci_echo_msg(uint8_t message_size, uint8_t *message_data);



bool lib_aci_radio_reset();

//@}

/** @} */

#endif /* LIB_ACI_H__ */
