/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 4808 $
 */ 

/** @file
@brief Implementation of a message dispatcher
*/

#include <stdlib.h> // malloc/free
#include <string.h>
#include "bool.h"
//#include "hal_power.h"
#include "lib_dispatcher.h"
#include "assertions.h"
#include "system.h"

#define DELAY_FREE_MSG 0xFF

#ifndef NULL
#define NULL (void*) 0
#endif

typedef struct lib_dispatcher_queue_t{
  uint8_t msg_handles[NB_MAX_EVT_PER_PRIORITY];
  uint8_t nb_msg_in_queue;
  uint8_t index_next_msg;
  uint8_t index_free_msg;
}lib_dispatcher_queue_t;

static volatile bool has_new_msg;

#ifndef DISPATCHER_WITH_SWITCH_CASE

  typedef struct lib_dispatcher_msg_t{
    #ifdef POST_WITH_DATA
      uint8_t size;
      uint8_t *msg_data;
    #endif //POST_WITH_DATA
    uint8_t nb_subscribers;
    dispatcher_cb_t cb_array[NB_MAX_SUBSCRIBER_PER_MSG];
  }lib_dispatcher_msg_t;


  static lib_dispatcher_msg_t dispather_tbl_array[] = MESSAGE_TABLE_INITIALIZER;

#else
  #ifdef POST_WITH_DATA
    typedef struct lib_dispatcher_msg_t{
      uint8_t size;
      uint8_t *msg_data;
    }lib_dispatcher_msg_t;

    static lib_dispatcher_msg_t dispather_tbl_array[NB_MAX_MSG];
  #endif //POST_WITH_DATA

#endif



static lib_dispatcher_queue_t msg_queues[NB_PRIORITIES];

#ifdef POST_WITH_DATA
void lib_dispatcher_post_msg(uint8_t msg_handle, uint8_t *p_buffer, uint8_t size, lib_dispatcher_priority_t msg_priority) // removed "reentrant"
#else
void lib_dispatcher_post_msg(uint8_t msg_handle, lib_dispatcher_priority_t msg_priority) // removed "reentrant"
#endif //POST_WITH_DATA
{
  lib_dispatcher_queue_t *p_queue;
  bool previous_ea;

#if (DEBUG_ENABLE!=NO_TRACES)
  if (msg_handle >= NB_MAX_MSG)
  {
    LIB_TRACES_LOG_INFO_1VAR(INFO_CODE_LIB_DISPATCHER_INVALID_HANLER, msg_handle);
  }
#endif
  ASSERT(ERROR_CODE_LIB_DISPATCHER_INVALID_HANDLE, (msg_handle < NB_MAX_MSG));

  if (msg_handle < NB_MAX_MSG)
  {
    has_new_msg = true;
    p_queue = &(msg_queues[msg_priority]);
    ASSERT(ERROR_CODE_LIB_DISPATCHER_MSG_OVERFLOW, (p_queue->nb_msg_in_queue < NB_MAX_EVT_PER_PRIORITY));
#ifdef POST_WITH_DATA    
    //dispather_tbl_array[msg_handle].msg_data = (uint8_t*) malloc(size*sizeof(uint8_t));
    //memcpy(dispather_tbl_array[msg_handle].msg_data, p_buffer, size);
    dispather_tbl_array[msg_handle].msg_data = p_buffer;
    dispather_tbl_array[msg_handle].size = size;
#endif //POST_WITH_DATA


    previous_ea = ARE_INTERRUPTS_ENABLED();
    DISABLE_INTERRUPTS();                           /*disable interrupts to protects the modification of index_free_msg*/
    p_queue->msg_handles[p_queue->index_free_msg] = msg_handle;
    p_queue->index_free_msg ++;
    if (previous_ea)
    {
      ENABLE_INTERRUPTS();                         /*eventually re-enable the interrupts if they were enabled*/
    }

    if (p_queue->index_free_msg == NB_MAX_EVT_PER_PRIORITY)
    {
      p_queue->index_free_msg = 0;
    }
    p_queue->nb_msg_in_queue ++;
  }
}



static void dispatch_msg(lib_dispatcher_queue_t *p_queue_to_dispatch) // removed "reentrant"
{
  uint8_t cur_msg_handle;
#ifdef POST_WITH_DATA
  uint8_t loc_size;
  uint8_t *p_loc_msg_data;
#endif //POST_WITH_DATA
  cur_msg_handle = p_queue_to_dispatch->msg_handles[p_queue_to_dispatch->index_next_msg];
  p_queue_to_dispatch->index_next_msg ++;
  if (p_queue_to_dispatch->index_next_msg == NB_MAX_EVT_PER_PRIORITY)
  {
    p_queue_to_dispatch->index_next_msg = 0;
  }
  p_queue_to_dispatch->nb_msg_in_queue --;

  ASSERT(ERROR_CODE_LIB_DISPATCHER_INVALID_HANDLE, (cur_msg_handle < NB_MAX_MSG));

  if (cur_msg_handle < NB_MAX_MSG)
  {
#ifdef POST_WITH_DATA
    loc_size = dispather_tbl_array[cur_msg_handle].size;
    p_loc_msg_data = dispather_tbl_array[cur_msg_handle].msg_data;
#endif //POST_WITH_DATA
    dispatch_msg_macro();
    //if (p_loc_msg_data != NULL)
    	//free(p_loc_msg_data);
  }
}

void lib_dispatcher_init()
{
  uint8_t queue_idx, handle_idx;
  
  for(queue_idx = 0; queue_idx < NB_PRIORITIES; queue_idx++)
  {
    for (handle_idx = NB_MAX_EVT_PER_PRIORITY-1; handle_idx > 0; handle_idx--)
    {
      msg_queues[queue_idx].msg_handles[handle_idx] = 0;
    }
    msg_queues[queue_idx].index_next_msg = 0;
    msg_queues[queue_idx].nb_msg_in_queue = 0;
    msg_queues[queue_idx].index_free_msg = 0;
  }
  has_new_msg = false;
#ifdef DISPATCHER_WITH_SWITCH_CASE
#ifdef POST_WITH_DATA
  {
    uint8_t i;
    for(i= 0; i < NB_MAX_MSG; i++)
    {
        dispather_tbl_array[i].msg_data = NULL;
        dispather_tbl_array[i].size = 0;
    }
  }
#endif //POST_WITH_DATA
#endif //DISPATCHER_WITH_SWITCH_CASE
}

void lib_dispatcher_dispatch(void) // removed "reentrant"
{
  uint8_t queue_idx;
  for(queue_idx = 0; queue_idx < NB_PRIORITIES; queue_idx++)
  {
    if (0 != msg_queues[queue_idx].nb_msg_in_queue)
    {
      has_new_msg = false;
      dispatch_msg(&(msg_queues[queue_idx]));
      return;
    }
  }

  //DISABLE_INTERRUPTS();
  if (!has_new_msg)
  {
    //hal_power_sleep();
	  __WFI();
  }
  //else
  //{
    //ENABLE_INTERRUPTS();
  //}

  /* After the sleep, the device will wake-up here */
}

